@@ -180,7 +180,7 @@ class DataLakeDirectoryClient(PathClient, DataLakeDirectoryClientBase):
                 :dedent: 4
                 :caption: Delete directory.
         """
-        return await self._delete(**kwargs)
+        return await self._delete(recursive=True, **kwargs)
 
     async def get_directory_properties(self, **kwargs):
         # type: (**Any) -> DirectoryProperties
@@ -506,6 +506,7 @@ class DataLakeDirectoryClient(PathClient, DataLakeDirectoryClientBase):
         )
         return DataLakeFileClient(
             self.url, self.file_system_name, file_path=file_path, credential=self._raw_credential,
+            api_version=self.api_version,
             _hosts=self._hosts, _configuration=self._config, _pipeline=self._pipeline,
             _location_mode=self._location_mode, require_encryption=self.require_encryption,
             key_encryption_key=self.key_encryption_key,
@@ -545,6 +546,7 @@ class DataLakeDirectoryClient(PathClient, DataLakeDirectoryClientBase):
         )
         return DataLakeDirectoryClient(
             self.url, self.file_system_name, directory_name=subdir_path, credential=self._raw_credential,
+            api_version=self.api_version,
             _hosts=self._hosts, _configuration=self._config, _pipeline=self._pipeline,
             _location_mode=self._location_mode, require_encryption=self.require_encryption,
             key_encryption_key=self.key_encryption_key,
