@@ -3,12 +3,14 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
-from typing import TYPE_CHECKING
+from typing import (  # pylint: disable=unused-import
+    Union, Optional, Any, TYPE_CHECKING
+)
 
-from azure.multiapi.storagev2.blob.v2019_07_07 import generate_account_sas as generate_blob_account_sas
-from azure.multiapi.storagev2.blob.v2019_07_07 import generate_container_sas, generate_blob_sas
+from azure.multiapi.storagev2.blob.v2020_10_02 import generate_account_sas as generate_blob_account_sas
+from azure.multiapi.storagev2.blob.v2020_10_02 import generate_container_sas, generate_blob_sas
 if TYPE_CHECKING:
-    import datetime
+    from datetime import datetime
     from ._models import AccountSasPermissions, FileSystemSasPermissions, FileSasPermissions, ResourceTypes, \
         UserDelegationKey
 
@@ -106,7 +108,7 @@ def generate_file_system_sas(
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
-        Permissions must be ordered read, write, delete, list.
+        Permissions must be ordered racwdlmeop.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
@@ -208,7 +210,7 @@ def generate_directory_sas(
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
-        Permissions must be ordered read, write, delete, list.
+        Permissions must be ordered racwdlmeop.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
@@ -317,7 +319,7 @@ def generate_file_sas(
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
-        Permissions must be ordered read, write, delete, list.
+        Permissions must be ordered racwdmeop.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
